# ifndef EvaluationFeature_h
# define EvaluationFeature_h
# include <Game/Evaluation.h>
namespace Alice
{
  class Game;
  class EvaluationFeature
  {
    enum{dimensions = 3};
  public:
    typedef Evaluation::Score Score;
    EvaluationFeature( int numberOfWeights, const char* fileName );
    virtual Evaluation::Score score(Game* g);
    void addDifference(float factor);
    void adjustWeights();
    void print();
    Score squaredNorm() const;
    virtual void computeGradient( Game* g);
    void prepare(Game* g);
  protected:
    void clearGradient();
    void setGradient( int i, Evaluation::Score x )
    {
      gradient[3*i]   = x * poly1(totalMaterial);
      gradient[3*i+1] = x * poly2(totalMaterial);
      gradient[3*i+2] = x * poly3(totalMaterial);
    };
    void addGradient( int i, Evaluation::Score x )
    {
      gradient[3*i]   += x * poly1(totalMaterial);
      gradient[3*i+1] += x * poly2(totalMaterial);
      gradient[3*i+2] += x * poly3(totalMaterial);
    };
    static Score poly1( Score x );
    static Score poly2( Score x );
    static Score poly3( Score x );
    std::vector<Evaluation::Score> weights;
    std::vector<Evaluation::Score> gradient;
    int totalMaterial;
  private:
    std::vector<Evaluation::Score> difference;
    const char* mFileName;
    bool couldRead;
  }; // class EvaluationFeature
}; // namespace Alice

# endif // EvaluationFeature_h
