# include <Game/Game.h>
# include <Game/EvaluationFeature.h>
# include <fstream>
# include <numeric>
using namespace Alice;

EvaluationFeature::EvaluationFeature(int numberOfWeights,
				     const char* fileName)
  : weights(numberOfWeights*dimensions),
    gradient(numberOfWeights*dimensions),
    difference(numberOfWeights*dimensions, 0),
    mFileName(fileName),
    couldRead(false)
{
  std::ifstream file(fileName);
  if (!file)
    {
      
      return;
    }
  std::string buffer;
  file >> buffer; // "#"
  file >> buffer; // "Alice"
  file >> buffer; // "evaluation"
  file >> buffer; // "feature"
  file >> buffer; // "version"
  int version;
  file >> version;
  couldRead = true;
  if (version == 1)
    for (int i = 0; i < numberOfWeights; i++)
      {
	file >> weights[3*i] ;
	weights[3*i+1] = weights[3*i];
	weights[3*i+2] = weights[3*i];
      }
  else
    for (int i = 0; i < weights.size(); i++)
      file >> weights[i];
};

void
EvaluationFeature::addDifference( float factor )
{
  for (int i = 0; i < gradient.size(); i++)
    difference[i] += gradient[i] * factor;
};

void
EvaluationFeature::adjustWeights( )
{
  for (int i = 0; i < weights.size(); i++)
    weights[i] += difference[i];
  std::ofstream file(mFileName);
  if (! file)
    return;
  file << "# Alice evaluation feature version 2"<<std::endl;
  for (int i = 0; i < weights.size(); i++)
    file << weights[i]<<std::endl;
};

void
EvaluationFeature::print()
{
  for (int i = 0; i < weights.size(); i++)
    std::cout<<weights[i]<<" ";
  std::cout<<std::endl;
};

EvaluationFeature::Score
EvaluationFeature::squaredNorm() const
{
  return std::inner_product(gradient.begin(), gradient.end(),
			    gradient.begin(), .0f);
};


void
EvaluationFeature::prepare(Game* g)
{
  totalMaterial = g->board().totalMaterial();
}

Evaluation::Score
EvaluationFeature::score(Game* g)
{
  prepare( g );
  computeGradient( g );
  return std::inner_product(weights.begin(), weights.end(),
			    gradient.begin(), Evaluation::Score(0));
};

void
EvaluationFeature::computeGradient( Game* g )
{

};

void
EvaluationFeature::clearGradient()
{

  for (int i = 0; i < gradient.size(); i++)
    gradient[i] = 0;
};

Evaluation::Score
EvaluationFeature::poly1(Score x ) 
{
  return (x-39)*(x-78)/((0-39)*(0-78));
};

Evaluation::Score
EvaluationFeature::poly2(Score x ) 
{
  return (x-0)*(x-78)/((39-0)*(39-78));
};


Evaluation::Score
EvaluationFeature::poly3(Score x ) 
{
  return (x-0)*(x-39)/((78-0)*(78-39));
};
