//////////////////////////////////////////////////
// (c) MMV Sven Reichard
// 

# include <OpeningBook.h>
# include <fstream>
# include <cmath>
#include <cstdlib>

OpeningBook::OpeningBook()
  :  current( &root ),
     inBook(true)
{

};

OpeningBook::~OpeningBook()
{

};

bool
OpeningBook::open(const char* filename )
{
  std::ifstream stream(filename);
  if (!stream) return false;
  root.readFrom(stream);
  return true;
};

bool
OpeningBook::save(const char* filename)
{
  std::ofstream stream(filename);
  if (!stream) return false;
  root.printOn(stream);
  return true;
};

void
OpeningBook::makeMove( const std::string& move )
{
  OpeningBookNode* child = current->first;
  while (child && (child->move != move))
    child = child->next;
  if (child)
    {
      current = child;
      return;
    }
  if (inBook)
    {
      std::ofstream logFile("opening.log", std::ios_base::app);
      logFile<<"tellics whisper Out of book."<<std::endl;
    }
  inBook = false;
  OpeningBookNode* newNode = new OpeningBookNode;
  newNode->move = move;
  current->addFirst(newNode);
  if (current->hasAlternative && move == current->alternative)
    {
      current->alternative = "";
      current->hasAlternative = 0;
    }
  current = newNode;
  current->updateAll();
};

std::string
OpeningBook::suggestMove() const
{
  std::ofstream logFile("opening.log", std::ios_base::app);
  if (current->isWon)
    {
      logFile<<"tellics whisper Bookvalue: won position"<<std::endl;
      OpeningBookNode* child = current->first;
      int wonChildren = 0;
      logFile<<"tellics whisper Options: ";
      for (; child; child = child->next)
	if (!child->isWon)
	  {
	    wonChildren ++;
	    logFile<<child->move<<" ";
	  }
      logFile<<std::endl;
      if (wonChildren == 0)
	return "-";
      double ran = 1.0 *wonChildren* std::rand() / RAND_MAX;
      int r = floor(ran);
      if (r >= wonChildren) r = wonChildren -1;
      r++;
      for (child = current->first; r&&child;child = child->next)
	if (!child->isWon)
	  if (! --r)
	    break;
      if (child)
	{
	  logFile<<"tellics whisper Choosing: "<<child->move<<std::endl;
	  return child->move;
	}
      return "-";
    }
  Evaluation::Score wanted = -current->treeValue;
  OpeningBookNode* child = current->first;
  while(child && child->opponentValue != wanted)
    child = child->next;
  if (child)
    {
      logFile<<"tellics whisper bookvalue: "<<current->treeValue<<std::endl;
      logFile<<"tellics whisper Playing: "<<child->move<<std::endl;
      return child->move;
    }
  if (current->hasAlternative)
    {
      logFile<<"tellics whisper playing novelty "
	       <<current->alternative<<std::endl;
      return current->alternative;
    }
  return "-";
};

int
OpeningBook::findOneAlternative(double seconds)
{
  for (int level = 0; level < 70; level++)
    if (  root.findOneAlternative(level, seconds))
      return level;
  return -1;
};
