//////////////////////////////////////////////////
// (c) MMV Sven Reichard
// 

# include <OpeningBookNodeTest.h>
# include <Moves/Move.h>
# include <SmartPointer.h>
CPPUNIT_TEST_SUITE_REGISTRATION( OpeningBookNodeTest );

void
OpeningBookNodeTest::testConstructor()
{
  OpeningBookNode x;
  CPPUNIT_ASSERT_DOUBLES_EQUAL( Evaluation::mate(), x.treeValue, 1e-6 );
  CPPUNIT_ASSERT_DOUBLES_EQUAL( -Evaluation::mate(), x.opponentValue, 1e-6 );
  CPPUNIT_ASSERT( !x.isWon );
};

void
OpeningBookNodeTest::testPointers()
{
  OpeningBookNode x;
  OpeningBookNode* null = 0;
  CPPUNIT_ASSERT_EQUAL( null, x.parent );
  CPPUNIT_ASSERT_EQUAL( null, x.next );
  CPPUNIT_ASSERT_EQUAL( null, x.first );
};

void
OpeningBookNodeTest::testAddNode()
{
  OpeningBookNode* x = new OpeningBookNode;
  OpeningBookNode* y = new OpeningBookNode;
  x->addFirst( y );
  CPPUNIT_ASSERT_EQUAL( y, x->first);
  CPPUNIT_ASSERT_EQUAL( x, y->parent);
  OpeningBookNode* z = new OpeningBookNode;
  x->addFirst( z );
  CPPUNIT_ASSERT_EQUAL( z, x->first );
  CPPUNIT_ASSERT_EQUAL( y, z->next );
  delete x;
};


void
OpeningBookNodeTest::testUpdate()
{
  OpeningBookNode* x = new OpeningBookNode;
  OpeningBookNode* y = new OpeningBookNode;
  OpeningBookNode* z = new OpeningBookNode;
  y->isWon = true;
  z->isWon = true;
  x->addFirst(y);
  x->addFirst(z);
  x->update();
  CPPUNIT_ASSERT( !x->isWon );
  y->isWon = false;
  x->update();
  CPPUNIT_ASSERT( x->isWon );
  z->isWon = false;
  x->update();
  CPPUNIT_ASSERT( x->isWon );
  
};


void
OpeningBookNodeTest::testUpdateAll()
{
  OpeningBookNode* x = new OpeningBookNode;
  OpeningBookNode* y = new OpeningBookNode;
  x->addFirst(y);
  y->updateAll();
  CPPUNIT_ASSERT( x->isWon );
};

void
OpeningBookNodeTest::testAlternativeValue()
{
  OpeningBookNode* x = new OpeningBookNode;
  OpeningBookNode* y = new OpeningBookNode;
  OpeningBookNode* z = new OpeningBookNode;
  x->addFirst(y);
  z->addFirst(x);
  CPPUNIT_ASSERT_DOUBLES_EQUAL(Evaluation::mate(), x->alternativeValue, 1E-6);
  y->alternativeValue = -1.0;
  y->updateAll();
  CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, x->opponentValue, 1e-6);
  CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.0, z->treeValue, 1e-6);
};

void
OpeningBookNodeTest::testSetAlternative()
{
  OpeningBookNode x;
  CPPUNIT_ASSERT_EQUAL( std::string(""), x.alternative );
  CPPUNIT_ASSERT( ! x.hasAlternative );
  x.setAlternative("e4", 5.0);
  CPPUNIT_ASSERT_EQUAL( std::string("e4"), x.alternative );
  CPPUNIT_ASSERT_DOUBLES_EQUAL( 5.0, x.alternativeValue, 1e-6 );
  CPPUNIT_ASSERT( x.hasAlternative );
};

